# -*- coding: utf-8 -*-
#
# File: testUtils.py
#
# GNU General Public License (GPL)
#

from AccessControl import Unauthorized
from collective.contact.plonegroup.utils import get_plone_group
from ftw.labels.interfaces import ILabeling
from imio.helpers.content import richtextval
from os import path
from plone.app.controlpanel.events import ConfigurationChangedEvent
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.permissions import View
from Products.CMFPlone.utils import safe_unicode
from Products.PloneMeeting.config import EXECUTE_EXPR_VALUE
from Products.PloneMeeting.ftw_labels.utils import get_labels
from Products.PloneMeeting.tests.PloneMeetingTestCase import PloneMeetingTestCase
from Products.PloneMeeting.utils import duplicate_portal_type
from Products.PloneMeeting.utils import escape
from Products.PloneMeeting.utils import isPowerObserverForCfg
from Products.PloneMeeting.utils import org_id_to_uid
from Products.PloneMeeting.utils import sendMail
from Products.PloneMeeting.utils import sendMailIfRelevant
from Products.PloneMeeting.utils import set_dx_value
from Products.PloneMeeting.utils import set_field_from_ajax
from Products.PloneMeeting.utils import transformAllRichTextFields
from Products.PloneMeeting.utils import validate_item_assembly_value
from zope.event import notify
from zope.schema._bootstrapinterfaces import WrongType


ASSEMBLY_CORRECT_VALUE = u'[[Text]][[Text]]'
ASSEMBLY_WRONG_VALUE = u'[[Text Text'


class testUtils(PloneMeetingTestCase):
    '''Tests the utils methods.'''

    def test_pm_Validate_item_assembly_value(self):
        """This will check the itemAssembly validity regarding [[ and ]]."""
        # empty value
        self.assertTrue(validate_item_assembly_value(u''))
        # correct values
        self.assertTrue(validate_item_assembly_value(ASSEMBLY_CORRECT_VALUE))
        self.assertTrue(validate_item_assembly_value(u'[[Text]] Text Text [[Text]]'))
        self.assertTrue(validate_item_assembly_value(u'[[Text]] Text Text [[Text]]'))
        self.assertTrue(validate_item_assembly_value(u'Text Text Text [[Text]]'))
        self.assertTrue(validate_item_assembly_value(u'[[Text]] Text Text Text'))
        self.assertTrue(validate_item_assembly_value(u'Text Text [[Text]] Text'))
        # wrong values
        self.assertFalse(validate_item_assembly_value(ASSEMBLY_WRONG_VALUE))
        self.assertFalse(validate_item_assembly_value(u'[[Text [[Text'))
        self.assertFalse(validate_item_assembly_value(u'[[Text [[Text'))
        self.assertFalse(validate_item_assembly_value(u']]Text [[Text'))
        self.assertFalse(validate_item_assembly_value(u'Text [[Text'))
        self.assertFalse(validate_item_assembly_value(u'Text Text]]'))

        # we have a special case, if REQUEST contains 'initial_edit', then validation
        # is bypassed, this let's edit an old wrong value
        self.request.set('initial_edit', u'1')
        self.assertTrue(validate_item_assembly_value(ASSEMBLY_WRONG_VALUE))

    def test_pm_Set_field_from_ajax(self):
        """Work on AT and DX."""
        cfg = self.meetingConfig
        cfg.setItemAdviceStates((self._stateMappingFor('proposed'), ))
        cfg.setItemAdviceEditStates((self._stateMappingFor('proposed'), ))
        cfg.setItemAdviceViewStates((self._stateMappingFor('proposed'), ))

        # item
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        item.setOptionalAdvisers((self.vendors_uid, ))
        item._update_after_edit()
        new_value = "<p>My item description.</p>"
        self.assertEqual(item.Description(), "")
        self.assertFalse(self.catalog(Description="my item description"))
        set_field_from_ajax(item, 'description', new_value)
        self.assertEqual(item.Description(), new_value)
        self.assertEqual(self.catalog(Description="my item description")[0].UID, item.UID())
        self.assertEqual(self.catalog(SearchableText="my item description")[0].UID, item.UID())
        # will raise Unauthorized if user can not edit
        self.proposeItem(item)
        self.assertRaises(Unauthorized, set_field_from_ajax, item, 'description', new_value)

        # meeting
        self.changeUser('pmManager')
        meeting = self.create('Meeting')
        new_value = "<p>My meeting notes.</p>"
        self.assertIsNone(meeting.notes)
        self.assertFalse(self.catalog(SearchableText="my meeting notes"))
        set_field_from_ajax(meeting, 'notes', new_value)
        self.assertEqual(meeting.notes.output, new_value)
        self.assertEqual(self.catalog(SearchableText="my meeting notes")[0].UID, meeting.UID())
        # will raise Unauthorized if user can not edit
        self.closeMeeting(meeting)
        self.assertRaises(Unauthorized, set_field_from_ajax, meeting, 'notes', new_value)

        # advice
        self.changeUser('pmReviewer2')
        advice = self.addAdvice(item, advice_comment=u"")
        new_value = "<p>My advice comment.</p>"
        self.assertEqual(advice.advice_comment.raw, u"")
        self.assertFalse(self.catalog(SearchableText="my advice comment"))
        set_field_from_ajax(advice, 'advice_comment', new_value)
        self.assertEqual(advice.advice_comment.output, new_value)
        # will raise Unauthorized if user can not edit
        self.validateItem(item)
        self.assertRaises(Unauthorized, set_field_from_ajax, item, 'description', new_value)

    def test_pm_SendMailIfRelevant(self):
        """ """
        cfg = self.meetingConfig
        cfg.setMailMode("deactivated")
        self.changeUser('pmManager')
        item = self.create("MeetingItem", title="My item")
        params = {"obj": item,
                  "event": "itemPresented",
                  "value": "creators",
                  "isSuffix": True,
                  "debug": True}

        # deactivated
        self.assertIsNone(sendMailIfRelevant(**params))
        # enabled but not selected
        cfg.setMailMode("activated")
        self.assertIsNone(sendMailIfRelevant(**params))
        # enabled and selected
        cfg.setMailItemEvents(("itemPresented", ))
        recipients, subject, body = sendMailIfRelevant(**params)
        dev_creators = get_plone_group(self.developers_uid, 'creators')
        self.assertEqual(dev_creators.getMemberIds(),
                         ['pmCreator1', 'pmCreator1b', 'pmManager'])
        # not sent to action triggerer
        self.assertEqual(recipients,
                         [u'M. PMCreator One bee <pmcreator1b@plonemeeting.org>',
                          u'M. PMCreator One <pmcreator1@plonemeeting.org>'])
        self.assertEqual(
            subject,
            u"{0} - Item has been inserted into a meeting - My item".format(
                safe_unicode(cfg.Title())))
        self.assertEqual(
            body,
            u"This meeting may still be under construction and is potentially inaccessible.  "
            u"The item is entitled \"My item\". You can access this item here: {0}.".format(
                item.absolute_url()))

    def test_pm_SendMailIfRelevantIsGroupIds(self):
        """ """
        cfg = self.meetingConfig
        cfg.setMailMode("activated")
        cfg.setMailItemEvents(("item_state_changed_validate", ))
        # test also that custom state/transition title works
        self._updateItemValidationLevel(
            cfg,
            level=self._stateMappingFor('proposed'),
            state_title="New proposed title",
            leading_transition_title="New propose title")

        self.changeUser('pmManager')
        item = self.create("MeetingItem", title="My item")
        self.proposeItem(item)
        params = {"obj": item,
                  "event": "item_state_changed_validate",
                  "value": [self.developers_creators, self.vendors_creators],
                  "isGroupIds": True,
                  "debug": True}

        recipients, subject, body = sendMailIfRelevant(**params)
        # config wf state/transition title is correctly used
        self.assertTrue("New proposed title" in subject)
        self.assertTrue("New propose title" in subject)
        dev_creators = get_plone_group(self.developers_uid, 'creators')
        self.assertEqual(dev_creators.getMemberIds(),
                         ['pmCreator1', 'pmCreator1b', 'pmManager'])
        vendors_creators = get_plone_group(self.vendors_uid, 'creators')
        self.assertEqual(vendors_creators.getMemberIds(), ['pmCreator2'])
        # not sent to action triggerer
        self.assertEqual(recipients,
                         [u'M. PMCreator One bee <pmcreator1b@plonemeeting.org>',
                          u'M. PMCreator One <pmcreator1@plonemeeting.org>',
                          u'M. PMCreator Two <pmcreator2@plonemeeting.org>'])

    def test_pm_SendMailIfRelevantIsUserIds(self):
        """ """
        cfg = self.meetingConfig
        cfg.setMailMode("activated")
        cfg.setMailItemEvents(("item_state_changed_validate", ))

        self.changeUser('pmManager')
        item = self.create("MeetingItem", title="My item")
        params = {"obj": item,
                  "event": "item_state_changed_validate",
                  "value": ['pmObserver1', 'pmManager', 'pmCreator2'],
                  "isUserIds": True,
                  "debug": True}

        recipients, subject, body = sendMailIfRelevant(**params)
        # not sent to action triggerer
        self.assertEqual(recipients,
                         [u'M. PMObserver One <pmobserver1@plonemeeting.org>',
                          u'M. PMCreator Two <pmcreator2@plonemeeting.org>'])

    def _default_permission_mail_recipents(self):
        return [u'M. Budget Impact Editor <budgetimpacteditor@plonemeeting.org>',
                u'M. PMCreator One <pmcreator1@plonemeeting.org>',
                u'M. PMCreator One bee <pmcreator1b@plonemeeting.org>',
                u'M. PMObserver One <pmobserver1@plonemeeting.org>',
                u'M. PMReviewer One <pmreviewer1@plonemeeting.org>',
                u'M. Power Observer1 <powerobserver1@plonemeeting.org>',
                u'Site administrator <siteadmin@plonemeeting.org>']

    def _modify_permission_mail_recipents(self):
        return [u'M. PMCreator One <pmcreator1@plonemeeting.org>',
                u'M. PMCreator One bee <pmcreator1b@plonemeeting.org>',
                u'Site administrator <siteadmin@plonemeeting.org>']

    def test_pm_SendMailIfRelevantIsPermission(self):
        """ """
        cfg = self.meetingConfig
        cfg.setMailMode("activated")
        cfg.setMailItemEvents(("item_state_changed_validate", ))

        self.changeUser('pmManager')
        item = self.create("MeetingItem", title="My item")
        params = {"obj": item,
                  "event": "item_state_changed_validate",
                  "value": View,
                  "isPermission": True,
                  "debug": True}

        recipients, subject, body = sendMailIfRelevant(**params)
        # not sent to action triggerer
        self.assertEqual(sorted(recipients), self._default_permission_mail_recipents())
        # check for editors
        params["value"] = ModifyPortalContent
        recipients, subject, body = sendMailIfRelevant(**params)
        self.assertEqual(sorted(recipients), self._modify_permission_mail_recipents())

    def test_pm_SendMailMeetingConfigTitle(self):
        """Variable "meetingConfigTitle" used in mail subject will include
           MeetingConfig.groupConfig when relevant."""
        config_groups = (
            {'row_id': 'unique_id_1',
             'label': 'ConfigGroup1',
             'full_label': 'Config group 1'},
            {'row_id': 'unique_id_2',
             'label': 'ConfigGroup2',
             'full_label': 'Config group 2'},
            {'row_id': 'unique_id_3',
             'label': 'ConfigGroup3',
             'full_label': 'Config group 3'},
        )
        self.tool.setConfigGroups(config_groups)
        cfg = self.meetingConfig
        # "test" mailMode will return computed elements
        cfg.setMailMode('test')
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        obj, body, recipients, from_address, subject, attachments, translation_mapping = \
            sendMail([], item, '')
        # no config group, simple title
        self.assertEqual(translation_mapping['meetingConfigTitle'],
                         safe_unicode(cfg.Title()))
        # config group but different config title, simple title
        cfg.setConfigGroup('unique_id_3')
        obj, body, recipients, from_address, subject, attachments, translation_mapping = \
            sendMail([], item, '')
        self.assertEqual(translation_mapping['meetingConfigTitle'],
                         safe_unicode(cfg.Title()))
        # with several same config title, config group is preprended
        self.meetingConfig2.setTitle(cfg.Title())
        obj, body, recipients, from_address, subject, attachments, translation_mapping = \
            sendMail([], item, '')
        self.assertEqual(translation_mapping['meetingConfigTitle'],
                         u'Config group 3 - %s' % safe_unicode(cfg.Title()))
        # if "full_label" is empty, it is not preprended
        config_groups[-1]['full_label'] = ''
        self.tool.setConfigGroups(config_groups)
        obj, body, recipients, from_address, subject, attachments, translation_mapping = \
            sendMail([], item, '')
        self.assertEqual(translation_mapping['meetingConfigTitle'],
                         safe_unicode(cfg.Title()))

    def test_pm_org_id_to_uid(self):
        """Test the utils.org_id_to_uid function."""
        self.changeUser('pmManager')
        # org UID
        self.assertIsNone(org_id_to_uid(''))
        self.assertEqual(org_id_to_uid(self.developers.getId()), self.developers_uid)
        self.assertEqual(org_id_to_uid(self.vendors.getId()), self.vendors_uid)
        # org UID with suffix
        dev_id_creators = "{0}_creators".format(self.developers.getId())
        dev_uid_creators = "{0}_creators".format(self.developers.UID())
        self.assertEqual(org_id_to_uid(dev_id_creators), dev_uid_creators)
        ven_id_creators = "{0}_creators".format(self.vendors.getId())
        ven_uid_creators = "{0}_creators".format(self.vendors.UID())
        self.assertEqual(org_id_to_uid(ven_id_creators), ven_uid_creators)
        # raise_on_error
        self.assertRaises(KeyError, org_id_to_uid, "wrong/path")
        self.assertIsNone(org_id_to_uid("wrong/path", raise_on_error=False))

    def test_pm_duplicate_portal_type(self):
        """Test the utils.duplicate_portal_type function."""
        self.changeUser('siteadmin')
        new_portal_type = duplicate_portal_type("MeetingItem", "MeetingItemDummy")
        self.assertEqual(new_portal_type.id, "MeetingItemDummy")
        self.assertEqual(new_portal_type.title, "MeetingItemDummy")
        # there was a bug, categorized_elements from collective.iconifiedcategory
        # was computed on every IItem, and a portal_type is a IItem...
        self.assertFalse('categorized_elements' in new_portal_type.__dict__)

    def test_pm_escape(self):
        self.assertEqual(escape('Test < & > are replaced with HTML "entities"'),
                         'Test &lt; &amp; &gt; are replaced with HTML &quot;entities&quot;')
        self.assertEqual(escape('<h1>We have no respect for <em><strong>HTML tags</strong></em> either</h1>'),
                         '&lt;h1&gt;We have no respect for &lt;em&gt;&lt;strong&gt;'
                         'HTML tags&lt;/strong&gt;&lt;/em&gt; either&lt;/h1&gt;')

    def test_pm_GetMemberInfo(self):
        """Test portal_membership.getMemberInfo as is it monkeypatched to add caching."""
        pm = self.portal.portal_membership
        self.changeUser("pmManager")
        self.assertEqual(pm.getMemberInfo("pmManager")["fullname"], 'M. PMManager')
        self.member.setMemberProperties({"fullname": "M. PMManager New"})
        # still not changed as cache was not invalidated
        self.assertEqual(pm.getMemberInfo("pmManager")["fullname"], 'M. PMManager')
        notify(ConfigurationChangedEvent(self.portal, self.request))
        self.assertEqual(pm.getMemberInfo("pmManager")["fullname"], 'M. PMManager New')

    def test_pm_TransformAllRichTextFields(self):
        """Test that it does not alterate field content, especially
           links to internal content or image that uses resolveuid."""

        # MeetingItem AT
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        # add image
        file_path = path.join(path.dirname(__file__), 'dot.gif')
        file_handler = open(file_path, 'r')
        data = file_handler.read()
        file_handler.close()
        img_id = item.invokeFactory('Image', id='dot.gif', title='Image', file=data)
        img = getattr(item, img_id)

        # link to image using resolveuid
        text = '<p>Internal image <img src="resolveuid/{0}" />.</p>'.format(img.UID())
        item.setDescription(text)
        self.assertEqual(item.objectIds(), ['dot.gif'])
        transformAllRichTextFields(item)
        self.assertEqual(item.getRawDescription(), text)
        transformAllRichTextFields(item, onlyField="description")
        self.assertEqual(item.getRawDescription(), text)

        # Meeting DX
        self.changeUser('pmManager')
        meeting = self.create('Meeting')
        # add image
        img_id = meeting.invokeFactory('Image', id='dot.gif', title='Image', file=data)
        img = getattr(meeting, img_id)

        # link to image using resolveuid
        text = '<p>Internal image <img src="resolveuid/{0}" />.</p>'.format(img.UID())
        meeting.observations = richtextval(text)
        self.assertEqual(meeting.objectIds(), ['dot.gif'])
        transformAllRichTextFields(meeting)
        self.assertEqual(meeting.observations.raw, text)
        transformAllRichTextFields(meeting, onlyField="observations")
        self.assertEqual(meeting.observations.raw, text)

    def test_pm_Set_dx_value(self):
        """utils.set_dx_value will set a value on a DX content and check if current
           user has the permission and if the value does validate."""
        cfg = self.meetingConfig
        self._enableField('meeting_number', related_to='Meeting')
        self._removeConfigObjectsFor(cfg)
        self.changeUser('pmManager')
        meeting = self.create('Meeting')
        # if user able to set, data must validate
        # here first_item_number needs an integer
        self.assertEqual(meeting.first_item_number, -1)
        self.assertRaises(WrongType, set_dx_value, meeting, "first_item_number", "a")
        set_dx_value(meeting, "first_item_number", 50)
        self.assertEqual(meeting.first_item_number, 50)
        # can not do it if does not have the permission
        self.changeUser('pmCreator1')
        self.assertTrue(self.hasPermission(View, meeting))
        self.assertRaises(WrongType, set_dx_value, meeting, "first_item_number", "a")
        self.assertRaises(Unauthorized, set_dx_value, meeting, "first_item_number", 55)
        # parameter raise_unauthorized may be False
        set_dx_value(meeting, "first_item_number", 55, raise_unauthorized=False)
        # the value is not changed anyway
        self.assertEqual(meeting.first_item_number, 50)
        # make sure it is useable in TAL expressions
        self.changeUser('pmManager')
        cfg.setOnMeetingTransitionItemActionToExecute(
            [{'meeting_transition': 'freeze',
              'item_action': EXECUTE_EXPR_VALUE,
              'tal_expression':
                'python: pm_utils.set_dx_value(meeting, "meeting_number", 25)'}, ])
        self.assertEqual(meeting.meeting_number, -1)
        self.freezeMeeting(meeting)
        # if was initialized to "1" by doFreeze but onMeetingTransitionItemActionToExecute
        # did nothing because it is applied on each items and there are no items!
        # So when using it to update meeting we really need to make sure the work done
        # is only done one time
        self.assertEqual(meeting.meeting_number, 1)
        # set it back to -1 so we make sure onMeetingTransitionItemActionToExecute
        # is done after doFreeze
        meeting.meeting_number = -1
        self.do(meeting, 'backToCreated')
        item = self.create('MeetingItem', decision=self.decisionText)
        self.presentItem(item)
        self.freezeMeeting(meeting)
        self.assertEqual(meeting.meeting_number, 25)

    def test_pm_get_labels(self):
        """Test the ToolPloneMeeting.get_labels method
           that will return ftw.labels active_labels."""
        self._enableField('labels')
        self.changeUser("pmCreator1")
        item = self.create("MeetingItem")
        self.assertEqual(get_labels(item), {})
        labeling = ILabeling(item)
        labeling.update(['label'])
        labeling.pers_update(['suivi'], True)
        self.assertEqual(get_labels(item), {'label': 'Label', 'suivi': 'Suivi'})
        self.assertEqual(get_labels(item, False), {'label': 'Label'})
        self.assertEqual(get_labels(item, "only"), {'suivi': 'Suivi'})

    def test_pm_IsPowerObserverForCfg(self):
        """ """
        cfg = self.meetingConfig
        self.changeUser('pmManager')
        self.assertFalse(isPowerObserverForCfg(cfg))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['powerobservers']))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['restrictedpowerobservers']))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['powerobservers', 'restrictedpowerobservers']))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['unknown']))
        self.changeUser('powerobserver1')
        self.assertTrue(isPowerObserverForCfg(cfg))
        self.assertTrue(isPowerObserverForCfg(
            cfg, power_observer_types=['powerobservers']))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['restrictedpowerobservers']))
        self.assertTrue(isPowerObserverForCfg(
            cfg, power_observer_types=['powerobservers', 'restrictedpowerobservers']))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['unknown']))
        self.changeUser('restrictedpowerobserver1')
        self.assertTrue(isPowerObserverForCfg(cfg))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['powerobservers']))
        self.assertTrue(isPowerObserverForCfg(
            cfg, power_observer_types=['restrictedpowerobservers']))
        self.assertTrue(isPowerObserverForCfg(
            cfg, power_observer_types=['powerobservers', 'restrictedpowerobservers']))
        self.assertFalse(isPowerObserverForCfg(
            cfg, power_observer_types=['unknown']))


def test_suite():
    from unittest import makeSuite
    from unittest import TestSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testUtils, prefix='test_pm_'))
    return suite



# -*- coding: utf-8 -*-

from eea.facetednavigation.interfaces import ICriteria
from imio.helpers.catalog import reindexIndexes
from plone import api
from plone.app.contenttypes.migration.dxmigration import migrate_base_class_to_new_class
from plone.namedfile.file import NamedBlobFile
from Products.PloneMeeting.content.organization import PMOrganization
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator


class Migrate_To_4101(Migrator):

    def _updateFacetedFilters(self):
        """Update vocabulary used for "Taken over by".
           Make sure the default for contacts 'c5' widget is not a list."""
        logger.info("Updating faceted filter \"Taken over by\" for every MeetingConfigs...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            obj = cfg.searches.searches_items
            # update vocabulary for relevant filters
            criteria = ICriteria(obj)
            criteria.edit(
                'c12', **{
                    'vocabulary':
                        'Products.PloneMeeting.vocabularies.creatorswithnobodyforfacetedfiltervocabulary'})
            criteria.edit(
                'c27', **{
                    'vocabulary':
                        'Products.PloneMeeting.vocabularies.associatedgroupsvocabulary'})
        logger.info('Done.')

        logger.info("Updating faceted filter \"Defined in\" for 'c5' "
                    "criterion of contacts/orgs-searches...")
        obj = self.portal.contacts.get('orgs-searches')
        # as default is not correct (a string instead a list, we can not use edit or it fails to validate)
        criteria = ICriteria(obj)
        criterion = criteria.get('c5')
        criterion.default = u'collective.contact.plonegroup.interfaces.IPloneGroupContact'
        logger.info('Done.')

    def _updateSearchLastDecisionsQuery(self):
        """The searchlastdecisions collection query was updated to use the
           lasrt-decisions compound criterion adapter."""
        logger.info("Updating collection \"searchlastdecisions\" query for every MeetingConfigs...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            collection = cfg.searches.searches_decisions.searchlastdecisions
            # make sure the compound criterion adapter is used
            has_compound_adapter = bool([term for term in collection.query
                                         if term[u'i'] == u'CompoundCriterion'])
            if has_compound_adapter is False:
                query = collection.query
                query.append(
                    {'i': 'CompoundCriterion',
                     'o': 'plone.app.querystring.operation.compound.is',
                     'v': 'last-decisions'})
                collection.setQuery(list(query))
        logger.info('Done.')

    def _updateOrgsDashboardCollectionColumns(self):
        """Make sure the 'review_state' column is not displayed in dashboards displaying organizations."""
        logger.info("Updating dashboard organization collections to remove the \"review_state\" column...")
        orgs_searches_folder = self.portal.contacts.get('orgs-searches')
        # hide the 'review_state' column for orgs related collections
        for org_coll in orgs_searches_folder.objectValues():
            org_coll.customViewFields = [col_name for col_name in org_coll.customViewFields
                                         if col_name != u'review_state']
        logger.info('Done.')

    def _allowDashboardPODTemplateInDirectoryPortalType(self):
        """Add 'DashboardPODTemplate' to 'allowed_content_types' of 'directory' portal_type."""
        logger.info("Adding 'DashboardPODTemplate' to 'allowed_content_types' of 'directory' portal_type...")
        pType = self.portal.portal_types['directory']
        allowed_types = list(pType.allowed_content_types)
        if 'DashboardPODTemplate' not in allowed_types:
            allowed_types.append('DashboardPODTemplate')
            pType.allowed_content_types = allowed_types
        logger.info('Done.')

    def _addDashboardPODTemplateExportOrganizations(self):
        """Add the export organizations DashboardPODTemplate in the contacts directory."""
        logger.info("Adding 'Export CSV DashboardPODTemplate' to 'contacts' directory...")
        pod_template_id = 'export-organizations'
        contacts = self.portal.contacts
        if pod_template_id in contacts.objectIds():
            self._already_migrated()
            return

        profile_path = self.ps._getImportContext(self.profile_name)._profile_path
        odt_path = profile_path + '/../testing/templates/organizations-export.ods'
        odt_file = open(odt_path, 'rb')
        odt_binary = odt_file.read()
        odt_file.close()
        data = {'title': 'Export CSV',
                'pod_formats': ['ods', 'xls'],
                'dashboard_collections': contacts.get('orgs-searches').all_orgs.UID(),
                'odt_file': NamedBlobFile(
                    data=odt_binary,
                    contentType='application/vnd.oasis.opendocument.text',
                    # pt.odt_file could be relative (../../other_profile/templates/sample.odt)
                    filename=u'export-organizations.ods'),
                'use_objects': True,
                }
        podTemplate = api.content.create(
            id=pod_template_id,
            type='DashboardPODTemplate',
            container=contacts,
            **data)
        podTemplate.reindexObject()
        logger.info('Done.')

    def _removeSelectableForPlonegroupFieldOnOrganizations(self):
        '''Remove attribute 'selectable_for_plonegroup' on organizations.'''
        logger.info("Removing attribute 'selectable_for_plonegroup' on every organizations...")
        for brain in self.catalog(portal_type='organization'):
            org = brain.getObject()
            if hasattr(org, 'selectable_for_plonegroup'):
                delattr(org, 'selectable_for_plonegroup')
        logger.info('Done.')

    def _removeTagsParameterInCallToJSCallViewAndReloadInCloneToOtherMCActions(self):
        '''Parameter was remove from JS callViewAndReload, this is stored
           in clone to other mc actions on MeetingItem portal_types.'''
        logger.info("Calling registerPortalTypes for every MeetingConfigs...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            cfg.registerPortalTypes()
        logger.info('Done.')

    def _moveToMeetingConfigOnMeetingTransitionItemActionToExecute(self):
        '''Field MeetingConfig.onMeetingTransitionItemTransitionToTrigger was moved
           to MeetingConfig.onMeetingTransitionItemActionToExecute.'''
        logger.info("Migrating field onMeetingTransitionItemActionToExecute for every MeetingConfigs...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            if hasattr(cfg, 'onMeetingTransitionItemTransitionToTrigger'):
                old_values = cfg.onMeetingTransitionItemTransitionToTrigger
                # move 'item_transition' to 'item_action' and add empty 'tal_expression'
                new_values = []
                for old_value in old_values:
                    new_values.append({'meeting_transition': old_value['meeting_transition'],
                                       'item_action': old_value['item_transition'],
                                       'tal_expression': ''})
                cfg.setOnMeetingTransitionItemActionToExecute(new_values)
                delattr(cfg, 'onMeetingTransitionItemTransitionToTrigger')
        logger.info('Done.')

    def _migrateContactOrganizationPersonsKlass(self):
        """klass used by 'organization' portal_type changed, this is only relevant for
           users using beta versions..."""
        logger.info('Migrating klass of collective.contact Organization to PMOrganization...')
        for brain in self.catalog(portal_type='organization'):
            org = brain.getObject()
            if not isinstance(org, PMOrganization):
                logger.info('Migrating organization at {0}'.format('/'.join(org.getPhysicalPath())))
                migrate_base_class_to_new_class(
                    org,
                    old_class_name='collective.contact.core.content.organization.Organization',
                    new_class_name='Products.PloneMeeting.content.organization.PMOrganization')
        logger.info('Done.')

    def _correctAccessToPODTemplates(self):
        """Correct weird Unauthorized when accessing POD template file."""
        logger.info('Correcting access to POD Templates file...')
        brains = self.catalog(
            object_provides=(
                'collective.documentgenerator.content.pod_template.IPODTemplate',
                'collective.documentgenerator.content.style_template.IStyleTemplate'))
        for brain in brains:
            pod_template = brain.getObject()
            logger.info('Updating access for POD template at {0}'.format(
                '/'.join(pod_template.getPhysicalPath())))
            pod_template.reindexObject()
        logger.info('Done.')

    def run(self, extra_omitted=[], from_migration_to_41=False):
        logger.info('Migrating to PloneMeeting 4101...')
        self._updateFacetedFilters()
        self._updateSearchLastDecisionsQuery()
        self._updateOrgsDashboardCollectionColumns()
        self._allowDashboardPODTemplateInDirectoryPortalType()
        self._addDashboardPODTemplateExportOrganizations()
        self._removeSelectableForPlonegroupFieldOnOrganizations()
        self._removeTagsParameterInCallToJSCallViewAndReloadInCloneToOtherMCActions()
        self._moveToMeetingConfigOnMeetingTransitionItemActionToExecute()
        self._migrateContactOrganizationPersonsKlass()
        self._correctAccessToPODTemplates()
        self.cleanRegistries()
        # holidays 2020 were added
        self.updateHolidays()
        # refresh if we are not coming from migration to 4.1,
        # in this case it was already refreshed
        if not from_migration_to_41:
            self.reindexIndexesFor(idxs=['get_full_title'], **{'portal_type': ['organization']})
            reindexIndexes(self.portal, idxs=['getTakenOverBy', 'getConfigId'])
            # re-run the Meeting workflows update as there was a bug in Migrator.refreshDatabase
            meeting_wf_ids = self.getWorkflows(meta_types=['Meeting'])
            self.refreshDatabase(catalogs=False, workflows=True, workflowsToUpdate=meeting_wf_ids)
        self.tool.invalidateAllCache()


def migrate(context):
    '''This migration function will:

       1) Update faceted filters for item dashboards;
       2) Update the searchlastdecisions query to use last-decisions compound criterion adapter;
       3) Update columns of collections displaying organizations in dashboard, remove the 'review_state' column;
       4) Add 'DashboardPODTemplate' to the allowed types of a contacts directory;
       5) Add the 'Export CSV' DashboardPODTemplate available on the contacts 'All orgs' dashboard;
       6) Remove the 'selectable_for_plonegroup' attribute on organizations;
       7) Update 'clone to other MC' portal_type action;
       8) Move field MeetingConfig.onMeetingTransitionItemTransitionToTrigger to
          MeetingConfig.onMeetingTransitionItemActionToExecute;
       9) Migrate organizations having klass Organization to klass PMOrganization;
       10) Fix access to POD template file;
       11) Clean registries;
       12) Update holidays to manage 2020;
       13) Reindex catalog indexes :
              - 'getTakenOverBy' as we use an indexer to handle empty value;
              - 'getConfigId' as we store a specific empty value as it is not possible to search on an empty index;
              - 'get_full_title' as indexed value is different than displayed one.
       14) Refresh Meeting workflows so MeetingManager have 'Review portal content' in state 'closed';
       15) Invalidate all cache.
    '''
    migrator = Migrate_To_4101(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-

from collective.behavior.talcondition.behavior import ITALCondition
from collective.behavior.talcondition.interfaces import ITALConditionable
from collective.eeafaceted.collectionwidget.utils import _get_criterion
from collective.eeafaceted.collectionwidget.utils import getCollectionLinkCriterion
from collective.eeafaceted.dashboard.browser.facetedcollectionportlet import Renderer as FacetedRenderer
from eea.facetednavigation.widgets.sorting.widget import Widget as SortingWidget
from imio.helpers.cache import get_cachekey_volatile
from imio.helpers.cache import get_plone_groups_for_user
from plone import api
from plone.app.portlets.portlets import base
from plone.memoize import ram
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema
from zope.formlib import form
from zope.i18nmessageid import MessageFactory
from zope.interface import implements


_ = MessageFactory('PloneMeeting')


class ITodoPortlet(IPortletDataProvider):
    """
      A portlet that shows things to do
    """
    batch_size = schema.Int(
        title=_(u'Number of items to display'),
        description=_(u'How many items to list in each topic.'),
        required=True,
        default=3)
    title_length = schema.Int(
        title=_(u'Number of characters of the title to show'),
        description=_(u'Limit the number of shown characters of the title to avoid too " \
                      long titles displayed in the portlet.'),
        required=True,
        default=45)


class Assignment(base.Assignment):
    implements(ITodoPortlet)

    # this needs to be done for old portlets that did not have the new batch_size attribute
    batch_size = 3
    title_length = 100

    def __init__(self, batch_size=3, title_length=100):
        self.batch_size = batch_size
        self.title_length = title_length

    @property
    def title(self):
        return _(u"To do")


class Renderer(base.Renderer, FacetedRenderer):

    _template = ViewPageTemplateFile('templates/portlet_todo.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)

        self.portal = api.portal.get()
        self.tool = api.portal.get_tool('portal_plonemeeting')
        self.cfg = self.tool.getMeetingConfig(self.context)

    @property
    def available(self):
        """
          Defines if the portlet is available in the context
        """
        available = FacetedRenderer(
            self.context, self.request, self.view, self.manager, self.data).available
        return available and self.showTodoPortlet(self.context)

    def render_cachekey(method, self):
        '''cachekey method for self.__call__.'''
        # URL to the item can change if server URL changed
        server_url = self.request.get('SERVER_URL', None)
        # cache until an item is modified
        date = get_cachekey_volatile('Products.PloneMeeting.MeetingItem.modified')
        load_portlet_todo = self.request.get('load_portlet_todo', False)
        return (repr(self.cfg),
                get_plone_groups_for_user(),
                server_url,
                date,
                load_portlet_todo)

    @ram.cache(render_cachekey)
    def render(self):
        return self._template()

    def showTodoPortlet(self, context):
        '''Must we show the portlet_todo ?'''
        if not self.cfg:
            return False
        if self.cfg.getToDoListSearches() and self.getSearches():
            return True
        return False

    def getSearches_cachekey(method, self):
        '''cachekey method for self.getSearches.'''
        return get_plone_groups_for_user(), self.cfg.modified()

    @ram.cache(getSearches_cachekey)
    def getSearches(self):
        ''' Returns the list of searches to display in portlet_todo.'''
        res = []
        if not self.cfg:
            return res
        # add a special key 'fromPortletTodo' in the extra_expr_ctx specifying
        # that we are querying available searches from the portlet_todo,
        # this way, we can use a different condition to display search in the
        # portlet_todo, for example shown for everyone in the
        # portlet_plonemeeting but only for some users in the portlet_todo
        # or only shown in the portlet_todo
        data = {'fromPortletTodo': True, }

        for search in self.cfg.getToDoListSearches(theObjects=True):
            if ITALConditionable.providedBy(search):
                data.update({'obj': search})
                if not ITALCondition(search).evaluate(extra_expr_ctx=data):
                    continue
            res.append('/'.join(search.getPhysicalPath()))
        return res

    def doSearch(self, collection_path):
        """ """
        collection = self.portal.unrestrictedTraverse(collection_path)
        # get the sorting, either faceted sorting criterion is used
        # or we will use sort_on and sort_reversed defined on collection
        sorting_criterion = _get_criterion(
            self.cfg.searches.searches_items, SortingWidget.widget_type)
        if sorting_criterion and sorting_criterion.default:
            sort_on = sorting_criterion.default.split('(reverse)')[0]
        else:
            sort_on = collection.sort_on
        return collection.results(**{'limit': self.data.batch_size,
                                     'sort_on': sort_on}), collection

    def getPrettyLink(self, brain):
        """
          Get the colored link for current item.
          In some case, due to current roles changes, the brain.getObject
          will not work, that is why we use unrestricted getObject.
        """
        # received brain is a plone.app.contentlisting.catalog.CatalogContentListingObject instance
        brain = getattr(brain, '_brain', brain)
        return brain._unrestrictedGetObject().getPrettyLink(maxLength=self.data.title_length)

    def getCollectionWidgetId(self):
        """Returns the collection widget id to be used in the URL generated on the collection link."""
        criterion = getCollectionLinkCriterion(self._criteriaHolder)
        if criterion:
            return criterion.getId()
        return ''


class AddForm(base.AddForm):
    form_fields = form.Fields(ITodoPortlet)
    label = _(u"Add Todo Portlet")
    description = _(u"This portlet shows things to do.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    form_fields = form.Fields(ITodoPortlet)
    label = _(u"Edit Todo Portlet")
    description = _(u"This portlet shows things to do.")



# -*- coding: utf-8 -*-
#
# File: config.py
#
# GNU General Public License (GPL)
#

from collections import OrderedDict
from Products.CMFCore.permissions import setDefaultRoles
from zope.i18nmessageid import MessageFactory

import pkg_resources


PMMessageFactory = MessageFactory("PloneMeeting")

HAS_ZAMQP = True
try:
    pkg_resources.get_distribution('imio.zamqp.pm')
except pkg_resources.DistributionNotFound:
    HAS_ZAMQP = False

HAS_SOLR = True
try:
    pkg_resources.get_distribution('collective.solr')
except pkg_resources.DistributionNotFound:
    HAS_SOLR = False

HAS_RESTAPI = True
try:
    pkg_resources.get_distribution('plonemeeting.restapi')
except pkg_resources.DistributionNotFound:
    HAS_RESTAPI = False


HAS_LDAP = True
try:
    pkg_resources.get_distribution('plone.app.ldap')
except pkg_resources.DistributionNotFound:
    HAS_LDAP = False

__author__ = """Gaetan DELANNAY <gaetan.delannay@geezteem.com>, Gauthier BASTIEN
<g.bastien@imio.be>, Stephan GEULETTE <s.geulette@imio.be>"""
__docformat__ = 'plaintext'

PROJECTNAME = "PloneMeeting"

# Define PloneMeeting-specific permissions
ReadDecision = 'PloneMeeting: Read decision'
WriteDecision = 'PloneMeeting: Write decision'
ReadObservations = 'PloneMeeting: Read item observations'
WriteItemMeetingManagerFields = 'PloneMeeting: Write item MeetingManager reserved fields'
# this makes MeetingManager able to write fields using the WriteItemMeetingManagerFields
# permission, especially in tests as in the UI, the element is already created and has a WF state
setDefaultRoles(WriteItemMeetingManagerFields, ('MeetingManager', 'Manager',))
ReadBudgetInfos = 'PloneMeeting: Read budget infos'
WriteBudgetInfos = 'PloneMeeting: Write budget infos'
WriteInternalNotes = 'PloneMeeting: Write internal notes'
WriteCommitteeFields = 'PloneMeeting: Write committee fields'
WriteMarginalNotes = 'PloneMeeting: Write marginal notes'
WriteHarmlessConfig = 'PloneMeeting: Write harmless config'
WriteRiskyConfig = 'PloneMeeting: Write risky config'
AddAdvice = 'PloneMeeting: Add advice'
AddAnnex = 'PloneMeeting: Add annex'
AddAnnexDecision = 'PloneMeeting: Add annexDecision'
AddItem = 'PloneMeeting: Add MeetingItem'
AddMeeting = 'PloneMeeting: Add Meeting'
ManageOwnOrganizationFields = 'PloneMeeting: Manage internal organization fields'
ManageItemCategoryFields = 'PloneMeeting: Manage item category fields'
setDefaultRoles(ReadDecision, ('Manager',))
setDefaultRoles(WriteDecision, ('Manager',))
setDefaultRoles(AddAnnex, ('Manager',))
setDefaultRoles(AddAnnexDecision, ('Manager',))
setDefaultRoles(WriteMarginalNotes, ('Manager',))
# given to nobody by default, is only given on /contacts/plonegroup-organization folder
setDefaultRoles(ManageOwnOrganizationFields, ())
# given to nobody by default, is only given on .../meeting-config-id/categories
# and .../meeting-config-id/classifiers folder but not on .../meeting-config-id/meetingcategories
# this is done to hide some category fields for meeting categories
setDefaultRoles(ManageItemCategoryFields, ())
MEETING_REMOVE_MOG_WFA = 'meeting_remove_global_access'

# Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner', 'Contributor'))

ADD_CONTENT_PERMISSIONS = {
    'MeetingItem': AddItem,
    'Meeting': AddMeeting,
    'MeetingConfig': 'PloneMeeting: Manage configuration',
    'MeetingGroup': 'PloneMeeting: Manage configuration',
}

setDefaultRoles(AddItem, ('Manager', 'Editor', ))
setDefaultRoles(AddMeeting, ('Manager', ))
setDefaultRoles('PloneMeeting: Manage configuration', ('Manager', ))

product_globals = globals()

# Dependencies of Products to be installed by quick-installer
# override in custom configuration
DEPENDENCIES = []

# Dependend products - not quick-installed - used in testcase
# override in custom configuration
PRODUCT_DEPENDENCIES = []

# list of add content permissions for content added to Meeting base contents
ADD_SUBCONTENT_PERMISSIONS = [
    AddAdvice,
    AddAnnex,
    AddAnnexDecision,
    'ATContentTypes: Add Image']

# base suffixes, if necessary, use EXTRA_GROUP_SUFFIXES to extend it
# or monkeypatch it if base values are not correct
MEETING_GROUP_SUFFIXES = [
    {'fct_title': u'advisers',
     'fct_id': u'advisers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': True},
    {'fct_title': u'creators',
     'fct_id': u'creators',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': True},
    {'fct_title': u'observers',
     'fct_id': u'observers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': True},
    {'fct_title': u'prereviewers',
     'fct_id': u'prereviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': True},
    {'fct_title': u'reviewers',
     'fct_id': u'reviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': True},
    {'fct_title': u'level1reviewers',
     'fct_id': u'level1reviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': False},
    {'fct_title': u'level2reviewers',
     'fct_id': u'level2reviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': False},
    {'fct_title': u'level3reviewers',
     'fct_id': u'level3reviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': False},
    {'fct_title': u'level4reviewers',
     'fct_id': u'level4reviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': False},
    {'fct_title': u'level5reviewers',
     'fct_id': u'level5reviewers',
     'fct_orgs': [],
     'fct_management': False,
     'enabled': False},
]


# this is made to manage specific suffixes for a particular profile
# this will be like :
# [{'fct_title': u'additional_suffix',
#   'fct_id': u'additional_suffix',
#   'fct_orgs': ['path_to_group_id_1', 'path_to_group_id_2']},
# ]
EXTRA_GROUP_SUFFIXES = []

# This is the group created for each MeetingConfig where we store
# users that will be able to edit the budgetInfos field for items in state
# corresponding to MeetingConfig.itemBudgetInfosStates
BUDGETIMPACTEDITORS_GROUP_SUFFIX = 'budgetimpacteditors'
# This is the group created for each MeetingConfig where we store
# users that will receive the 'MeetingManager' role.  This group will be defined
# in localroles of every meetingConfig user folders (mymeetings/meetingconfigfolder)
# and on the corresponding MeetingConfig so these users are MeetingManager on the MeetingConfig too
MEETINGMANAGERS_GROUP_SUFFIX = 'meetingmanagers'
# This is the group created for each MeetingConfig where we store users able to manage item templates.
ITEMTEMPLATESMANAGERS_GROUP_SUFFIX = 'itemtemplatesmanagers'

# This is a mapping between usecases around the role Reader, so users that can see
# By default, the same role is used for different usecases, so it will give the same view permission by the wf
# If a special usecase needs to use another role, it can be specified in a sub-plugin
READER_USECASES = {
    'copy_groups': 'Reader',
    'restricted_copy_groups': 'Reader',
    'advices': 'Reader',
    'powerobservers': 'Reader',
    'itemtemplatesmanagers': 'Reader',
    'groupsincharge': 'Reader',
    'confidentialannex': 'AnnexReader',
}

# Suffixes that may evaluate MeetingItem.completeness
ITEM_COMPLETENESS_EVALUATORS = ('reviewers', 'prereviewers', )
# Suffixes that can ask new evaluation of MeetingItem.completeness if set to 'incomplete'
ITEM_COMPLETENESS_ASKERS = ('reviewers', 'prereviewers', 'creators', )

# The id used for the root folder added to the member personal area that
# will contain every meetingConfigs available to the member
ROOT_FOLDER = "mymeetings"
MEETING_CONFIG = "meeting_config"

TOOL_ID = "portal_plonemeeting"
TOOL_FOLDER_CATEGORIES = 'categories'
TOOL_FOLDER_CLASSIFIERS = 'classifiers'
TOOL_FOLDER_SEARCHES = 'searches'
TOOL_FOLDER_RECURRING_ITEMS = "recurringitems"
TOOL_FOLDER_ITEM_TEMPLATES = "itemtemplates"
TOOL_FOLDER_ANNEX_TYPES = 'annexes_types'
TOOL_FOLDER_POD_TEMPLATES = 'podtemplates'
TOOL_FOLDER_MEETING_CATEGORIES = 'meetingcategories'

ITEM_NO_PREFERRED_MEETING_VALUE = "whatever"

ITEM_DEFAULT_TEMPLATE_ID = "default-empty-item-template"

# default fields kept when an item is cloned
DEFAULT_COPIED_FIELDS = ['title', 'description', 'detailedDescription', 'motivation',
                         'decision', 'decisionSuite', 'decisionEnd',
                         'budgetInfos', 'budgetRelated', 'sendToAuthority',
                         'groupsInCharge', 'proposingGroupWithGroupInCharge',
                         'copyGroups', 'restrictedCopyGroups']
# extra fields kept when an item is cloned in the same meeting config,
# so not the case when sent to another meeting config
EXTRA_COPIED_FIELDS_SAME_MC = ['associatedGroups', 'category', 'classifier', 'committees',
                               'optionalAdvisers', 'otherMeetingConfigsClonableTo',
                               'otherMeetingConfigsClonableToPrivacy', 'oralQuestion',
                               'toDiscuss', 'privacy', 'pollType', 'textCheckList',
                               'otherMeetingConfigsClonableToFieldTitle',
                               'otherMeetingConfigsClonableToFieldDescription',
                               'otherMeetingConfigsClonableToFieldDetailedDescription',
                               'otherMeetingConfigsClonableToFieldMotivation',
                               'otherMeetingConfigsClonableToFieldDecision',
                               'otherMeetingConfigsClonableToFieldDecisionSuite',
                               'otherMeetingConfigsClonableToFieldDecisionEnd',
                               'isAcceptableOutOfMeeting']

EXTRA_COPIED_FIELDS_FROM_ITEM_TEMPLATE = ['observations', 'inAndOutMoves', 'notes',
                                          'internalNotes', 'committeeObservations',
                                          'committeeTranscript', 'votesObservations']

# to differenciate items of different meeting configs,
# use a different icon color (MeetingConfig.itemIconColor)
ITEM_ICON_COLORS = ("azur", "black", "green", "grey", "orange",
                    "pink", "purple", "red", "yellow")

NOT_ENCODED_VOTE_VALUE = u'not_yet'
NOT_VOTABLE_LINKED_TO_VALUE = u'not_votable_linked_to'
ALL_VOTE_VALUES = ('yes', 'no', 'abstain', 'does_not_vote', 'not_found', 'invalid', 'blank')
NOT_GIVEN_ADVICE_VALUE = 'not_given'
HIDDEN_DURING_REDACTION_ADVICE_VALUE = 'hidden_during_redaction'
CONSIDERED_NOT_GIVEN_ADVICE_VALUE = 'considered_not_given_hidden_during_redaction'
ADVICE_GIVEN_HISTORIZED_COMMENT = 'advice_given_was_modified_historized_comments'
ALL_ADVISERS_GROUP_VALUE = 'entireadvisersgroup'

# value displayed in the object history table if a comment is not viewable
HISTORY_COMMENT_NOT_VIEWABLE = "<span class='discreet'>Access to this comment is restricted.</span>"

# states in which advice WF is considered 'ended'
ADVICE_STATES_ENDED = ('advice_given', )
# to be monkey patched to extend it
# this is the mappings between an advice WF state and it's corresponding group suffix
ADVICE_STATES_MAPPING = {'advice_given': 'advisers', }

ADVICE_TYPES = [
    'positive',
    'positive_with_comments',
    'positive_with_remarks',
    'positive_after_modification',
    'cautious',
    'negative',
    'negative_with_remarks',
    'back_to_proposing_group',
    'nil',
    'read']

# name of the variable added to the REQUEST when getting the scan_id
ITEM_SCAN_ID_NAME = 'item_scan_id'

# Keys used in annotations
SENT_TO_OTHER_MC_ANNOTATION_BASE_KEY = 'PloneMeeting-sent_to_other_meetingconfig_'
CLONE_TO_OTHER_MC_ACTION_SUFFIX = 'clone_to_other_mc_'
CLONE_TO_OTHER_MC_EMERGENCY_ACTION_SUFFIX = 'clone_to_other_mc_emergency_'

# Value added in the CKeditor menuStyles to specify that it has been customized
CKEDITOR_MENUSTYLES_CUSTOMIZED_MSG = '/* Styles have been customized, do not remove this line! */'

# if a delay for giving an item is on saturday, we extends the delay to next avaialble day
# so here, we define that weekday 5 (as weekday starts from 0) is unavailble
DELAY_UNAVAILABLE_WEEKDAY_NUMBER = 5
# define the weekday mnemonics to match the date.weekday function
# take care that Zope DateTime sunday is weekday number is 0
# and python datetime sunday weekday number is 6...
PY_DATETIME_WEEKDAYS = ('mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun')

# default user password used when sample users are added to a PloneMeeting during install
DEFAULT_USER_PASSWORD = 'Meeting_12'

# columns that will be automatically selected for item related DashboardCollections
DEFAULT_ITEM_COLUMNS = ({'position': 0, 'name': 'pretty_link'},
                        {'position': 99, 'name': 'select_row'}, )
# columns that will be automatically selected for meeting related DashboardCollections
DEFAULT_MEETING_COLUMNS = ({'position': 0, 'name': 'pretty_link'},
                           {'position': 99, 'name': 'select_row'}, )

# default listTypes
DEFAULT_LIST_TYPES = [
    {'identifier': 'normal', 'label': 'normal', 'used_in_inserting_method': '1'},
    {'identifier': 'late', 'label': 'late', 'used_in_inserting_method': '1'}, ]

ITEM_LABELS_ACCESS_CACHE_ATTR = "_labels_access_cache"

NO_TRIGGER_WF_TRANSITION_UNTIL = '__nothing__'

AUTO_COPY_GROUP_PREFIX = 'auto__'

HIDE_DECISION_UNDER_WRITING_MSG = \
    "<p class='highlightValue'>The decision is currently under edit by managers, you can not access it.</p>"

DUPLICATE_EVENT_ACTION = 'Duplicate'
DUPLICATE_AND_KEEP_LINK_EVENT_ACTION = 'Duplicate and keep link'

# There are various ways to insert items into meetings
# each time we have the name of the inserting method and the name
# of the field managing values to compute it if any
ITEM_INSERT_METHODS = OrderedDict((
    # at the end of meetings;
    ('at_the_end', []),
    # depending on the item's listType, by default 'normal' or 'late';
    ('on_list_type', ['field_listTypes']),
    # according to category order;
    ('on_categories', ['category']),
    # according to classifier order;
    ('on_classifiers', ['classifier']),
    # according to proposing group order;
    ('on_proposing_groups', ['organization']),
    # according to all groups (among proposing group AND
    # associated groups). Similar to the previous sort method, with this
    # difference: the group taken into consideration is the group among all
    # groups that comes first in the order.
    ('on_all_groups', ['organization']),
    # according to the groupInCharge of the proposingGroup used for the item
    ('on_groups_in_charge', ['field_orderedGroupsInCharge', 'organization']),
    # according to the associatedGroups selected on the item
    # taking into account every selected associatedGroups
    # computing will generate following order :
    # items having associated group 1 only
    # items having associated group 1 and associated group 2
    # items having associated group 1 and associated group 2 and associated group 3
    # items having associated group 1 and associated group 2 and associated group 3 and associated group 4
    # items having associated group 1 and associated group 3
    # items having associated group 1 and associated group 3 and associated group 4
    ('on_all_associated_groups', ['field_orderedAssociatedOrganizations', 'organization']),
    # according to the committees
    ('on_all_committees', ['field_committees']),
    # according to the item privacy;
    ('on_privacy', ['field_selectablePrivacies']),
    # according to the item toDiscuss;
    ('on_to_discuss', []),
    # according to items that need to be sent to another meeting config;
    ('on_other_mc_to_clone_to', ['field_meetingConfigsToCloneTo']),
    # according to poll type;
    ('on_poll_type', ['field_usedPollTypes']),
    # alphabetically according to MeetingItem.title field content
    ('on_item_title', []),
    # alphabetically according to MeetingItem.decision field content first words
    ('on_item_decision_first_words', []),
    # alphabetically according to MeetingItem.Creator member fullname
    ('on_item_creator', []),
))

INSERTING_ON_ITEM_DECISION_FIRST_WORDS_NB = 5

ITEM_TRANSITION_WHEN_RETURNED_FROM_PROPOSING_GROUP_AFTER_CORRECTION = 'accept_but_modify'

EXECUTE_EXPR_VALUE = 'execute_tal_expression'

NO_COMMITTEE = u"no_committee"

ITEM_MOVAL_PREVENTED = "Prevented to rename item no more in initial_state!"

# for performance reason we do not dynamically get the annexes criterion id
FACETED_ANNEXES_CRITERION_ID = 'c20'

# name of marker specifyng that a reindex is required
REINDEX_NEEDED_MARKER = "_catalog_reindex_needed"
# prefix used in pm_technical_index to store item initiators
ITEM_INITIATOR_INDEX_PATTERN = "item_initiator_{0}"

# various place where attendees infos are stored on meeting
MEETING_ATTENDEES_ATTRS = (
    # place to store item absents
    'item_absents',
    # place to store item excused
    'item_excused',
    # place to store item non attendees
    'item_non_attendees',
    # place to store item signatories
    'item_signatories',
    # place to store item attendees changed position
    'item_attendees_positions',
    # place to store item attendees changed order
    'item_attendees_order',
    # place to store item votes
    'item_votes')


def registerClasses():
    '''ArchGenXML generated code does not register Archetype classes at the
       right moment since model adaptations have been implemented. This method
       allows to perform class registration at the right moment.'''
    from Products.Archetypes.atapi import registerType

    import Products.Archetypes
    global ADD_CONTENT_PERMISSIONS
    classNames = ADD_CONTENT_PERMISSIONS.keys()
    classNames.append('ToolPloneMeeting')
    for name in classNames:
        exec 'import Products.PloneMeeting.%s as module' % name
        klass = None  # PEP8
        exec 'klass = module.%s' % name
        key = 'PloneMeeting.%s' % name
        if key in Products.Archetypes.ATToolModule._types:
            # Unregister the class
            del Products.Archetypes.ATToolModule._types[key]
        registerType(klass, PROJECTNAME)


class PloneMeetingError(Exception):
    pass



# -*- coding: utf-8 -*-

from copy import deepcopy
from plone.app.controlpanel.mail import MailControlPanelAdapter
from Products.PloneMeeting.migrations import logger
from Products.PloneMeeting.migrations import Migrator
from Products.PloneMeeting.utils import get_public_url


class Migrate_To_4108(Migrator):

    def _correctDashboardCollectionsQuery(self):
        """Format of DashboardCollection query is sometimes broken, instead containing
           list of <dict>, it contains list of <instance> ???."""
        logger.info("Correcting query for DashboardCollections...")
        for cfg in self.tool.objectValues('MeetingConfig'):
            for subfolder in cfg.searches.objectValues():
                for search in subfolder.objectValues():
                    query = deepcopy(search.query)
                    query = [dict(crit) for crit in query]
                    search.setQuery(query)
        logger.info('Done.')

    def fix_email_from_address(self):
        logger.info("Fixing email from address...")
        mail_panel_adapter = MailControlPanelAdapter(self.portal)
        mail_address = mail_panel_adapter.get_email_from_address().strip()
        if "imio.be" in mail_address:
            public_url = get_public_url(self.portal)
            mail_address = public_url.replace("https://", "")\
                .replace("-pm.imio-app", "-delib@imio")
        mail_panel_adapter.set_email_from_address(mail_address.strip(" /."))
        logger.info('Done.')

    def run(self, from_migration_to_41=False):
        logger.info('Migrating to PloneMeeting 4108...')
        self._correctDashboardCollectionsQuery()
        # fix wrong condition for 'searchmyitemstakenover'
        # that used omittedSuffixes instead omitted_suffixes
        self.updateTALConditions(old_word='omittedSuffixes', new_word='omitted_suffixes')
        self.fix_email_from_address()


def migrate(context):
    '''This migration function will:

       1) Make sure format of DashboardCollection.query is correct;
       2) Fix condition for 'searchmyitemstakenover'.
       3) Fix mail sender address;
       4) Re-apply typeinfo step to update directory schema policy.
    '''
    migrator = Migrate_To_4108(context)
    migrator.run()
    migrator.finish()



# -*- coding: utf-8 -*-

from collective.eeafaceted.batchactions.interfaces import IBatchActionsMarker
from collective.eeafaceted.collectionwidget.interfaces import ICollectiveEeafacetedCollectionwidgetLayer
from collective.z3cform.datagridfield.interfaces import IDataGridFieldLayer
from ftw.labels.interfaces import ILabelSupport
from imio.actionspanel.interfaces import IActionsPanelLayer
from plone.dexterity.interfaces import IDexterityContent
from zope.component.interfaces import IObjectEvent
from zope.interface import Interface


class IAdvicesUpdatedEvent(IObjectEvent):
    """
    """


class IItemDuplicatedEvent(IObjectEvent):
    """
    """


class IItemDuplicatedToOtherMCEvent(IObjectEvent):
    """
    """


class IItemDuplicatedFromConfigEvent(IObjectEvent):
    """
    """


class IItemAfterTransitionEvent(IObjectEvent):
    """
    """


class IItemListTypeChangedEvent(IObjectEvent):
    """
    """


class IItemPollTypeChangedEvent(IObjectEvent):
    """
    """


class IItemLocalRolesUpdatedEvent(IObjectEvent):
    """
    """


class IMeetingAfterTransitionEvent(IObjectEvent):
    """
    """


class IMeetingLocalRolesUpdatedEvent(IObjectEvent):
    """
    """


class IAdviceAfterAddEvent(IObjectEvent):
    """
    """


class IAdviceAfterModifyEvent(IObjectEvent):
    """
    """


class IAdviceAfterTransitionEvent(IObjectEvent):
    """
    """


class IRedirect(Interface):
    """
    """
    def redirect():
        """
          Redirect to the right place in case we use plone.app.jquerytools overlays
        """


class IPloneMeetingLayer(ICollectiveEeafacetedCollectionwidgetLayer, IDataGridFieldLayer, IActionsPanelLayer):
    """
      Define a layer so some elements are only added for it.
      We inherit from other packages layers we want to be able to override.
    """
    pass


class IConfigElement(Interface):
    """Base marker interface for every config related elements
    """


class IPloneElement(Interface):
    """Base marker interface for every Plone default content types
    """


class IMeetingContent(Interface):
    """Base marker interface for every content related elements
    """


class IATMeetingContent(IMeetingContent):
    """Base marker interface for AT content related elements
    """


class IDXMeetingContent(IMeetingContent, IDexterityContent):
    """Base marker interface for DX content related elements
    """


class IMeetingItem(IATMeetingContent, ILabelSupport):
    """Marker interface for .MeetingItem.MeetingItem
    """


class IMeeting(IATMeetingContent):
    """Marker interface for .Meeting.Meeting
    """


class IMeetingItemDashboardBatchActionsMarker(IBatchActionsMarker):
    """Marker interfaces to register batch actions
       for dashboards displaying items."""


class IMeetingDashboardBatchActionsMarker(IBatchActionsMarker):
    """Marker interfaces to register batch actions
       for dashboards displaying meetings."""


class IMeetingBatchActionsMarker(IBatchActionsMarker):
    """Marker interfaces to register batch actions for Meetings."""


class IMeetingItemBatchActionsMarker(IBatchActionsMarker):
    """Marker interfaces to register batch actions for MeetingItems."""


class IMeetingAdviceBatchActionsMarker(IBatchActionsMarker):
    """Marker interfaces to register batch actions for MeetingAdvices."""


class IMeetingContentBatchActionsMarker(IBatchActionsMarker):
    """Marker interfaces to register batch actions for every MeetingContents."""


class IToolPloneMeeting(Interface):
    """Marker interface for .ToolPloneMeeting.ToolPloneMeeting
    """


class IMeetingCategory(IConfigElement):
    """Marker interface for .MeetingCategory.MeetingCategory
    """


class IMeetingConfig(IConfigElement):
    """Marker interface for .MeetingConfig.MeetingConfig
    """


class IMeetingGroup(IConfigElement):
    """Marker interface for .MeetingGroup.MeetingGroup
    """


class IMeetingUser(IConfigElement):
    """Marker interface for .MeetingUser.MeetingUser
    """


# Interfaces used for customizing the behaviour of meeting items ---------------
class IMeetingItemDocumentation:
    '''Normally, the methods described here should be part of IMeetingItem.
       Because it is impossible to do so with an overengineered yet overrigid
       ArchGenXML 2, we document the provided methods in this absurd class.'''
    def showObservations():
        '''Condition to show field observations, by default shown if in
           MeetingConfig.usedItemAttributes.'''
    def show_item_reference():
        '''When must I show the item reference ? In the default implementation,
           item references are shown as soon as a meeting is frozen or when
           using MeetingConfig.computeItemReferenceForItemsOutOfMeeting.'''
    def get_predecessors():
        '''Returns the entire chain of predecessors and successors.'''
    def addRecurringItemToMeeting(meeting):
        '''This meeting item was just created (by copy/pasting a recurring item)
           in the folder that also contains the p_meeting into which it must
           be inserted. So in this method, we must trigger automatically some
           transitions on the workflow defined for meeting items, in order
           to insert this item into the p_meeting and set it at the correct
           state.  If this method returns True, it means that an error occured
           while adding the recurring item to the meeting.'''
    def mayWriteCompleteness():
        '''This manage the condition that check if MeetingItem.complentess
           may be edited by the current user.'''
    def setHistorizedTakenOverBy(wf_state):
        '''Set 'takenOverBy' taking into account last user that was taking
           the item over.  So if an item come back a second time (or more), to
           the same p_wf_state, we automatically set the same user than before
           if still available.  If not, we set that to ''.'''
    def mayAskEmergency(self):
        '''Returns True if current user may ask emergency for an item.'''
    def mayAcceptOrRefuseEmergency(self):
        '''Returns True if current user may accept or refuse emergency if asked for an item.'''
    def mayEvaluateCompleteness(self):
        '''Condition for editing 'completeness' field,
           being able to define if item is 'complete' or 'incomplete'.'''
    def mayAskCompletenessEvalAgain(self):
        '''Condition for editing 'completeness' field,
           being able to ask completeness evaluation again when completeness
           was 'incomplete'.'''
    def _is_complete(self):
        '''Return True if item is complete when using MeetingItem.completeness.'''
    def mayTakeOver():
        '''This manage the condition that check if MeetingItem.takenOverBy
           may be edited by the current user.
           By default, item may be taken over :
           - if user has WF transitions to trigger;
           - if user is member of the proposing group (any subgroups).'''
    def transformRichTextField(fieldName, richContent):
        '''This method is called every time an item is created or updated. It
           allows you to modify the content of any "richtext" attribute
           (description, decision...) defined on the item (the field name is
           given in p_fieldName). The method must return the adapted version of
           p_richContent, which contains the XHTML content of the "richtext"
           field (a string). The default PloneMeeting behaviour for this method
           is to return p_richContent untouched.'''
    def onEdit(isCreated):
        '''This method is called every time an item is created or updated.
           p_isCreated is True if the object was just created. It is called
           within Archetypes methods at_post_create_script and
           at_post_edit_script. You do not need to reindex the item. The
           default PloneMeeting implementation for this method does nothing.'''
    def getCustomAdviceMessageFor(self, advice):
        '''This manages custom messages displayed on the advice infos tooltipster.
           If 'displayDefaultComplementaryMessage' is True, default message are displayed.
           If 'displayAdviceReviewState' is True, the advice review_state is displayed
           to users not able to see the advice.
           'customAdviceMessage' will contain the translated custom message.'''
    def mayCloneToOtherMeetingConfig(destMeetingConfigId):
        '''Check that we can clone the item to p_destMeetingConfigId.
           Checks are ordered from light to heavy as this could be called
           several times...'''
    def onDiscussChanged(toDiscuss):
        '''This method is called when value of field "toDiscuss" (p_toDiscuss)
           has changed on an item.'''
    def isPrivacyViewable():
        '''Privacy acts as a simple filter in front of workflow-based security.
           It means that, if someone has the "View" permission on an item, he
           may only access some information about the item. Field-specific
           permissions exists for managing this problem, but on a too detailed
           level (the field). Managing lots of field-specific permissions is
           laborious, unmaintanable and produces huge workflow descriptions.
           The idea here is to get an intermediate security filter. Very simple:
           people that have View permission on an item where
           item.isPrivacyViewable is False, will be able to consult information
           shown in lists of items (titles, descriptions, decisions), excepted
           advices and annexes, and will not be able to go to meetingitem_view.

           The default implementation: isPrivacyViewable is True for any user
           belonging to any of the Plone groups related to the MeetingGroup
           which is the proposing group for the item.
           Copy groups and advisers will also be able to access the item.
           Note that isPrivacyViewable is automatically True if the item has
           field privacy=False.'''
    def getExtraFieldsToCopyWhenCloning():
        '''While cloning an item (to another MeetingConfig or not), some fields are selected
           to be copied to the new cloned items.  If we want to add some arbitrary
           fields like fields coming from a SchemaExtender, we can specify this
           with this method that will returns a list of extra field ids to copy to
           the cloned item.  The parameter p_cloned_to_same_mc is True if current item
           will be cloned to the same meeting config, and is False if item is
           actually sent to another meeting config.  The parameter p_cloned_from_item_template
           is True if we are actually creating an item from an item template.'''
    def mayEditAdviceConfidentiality():
        '''Condition for being able to edit the confidentiality of asked advices.
           By default, only MeetingManagers able to edit the item may change advice confidentiality.'''
    def _itemToAdviceIsViewable(groupId):
        '''Is the item viewable by given p_groupId for which advice has been asked?'''
    def _adviceIsAddable(groupId):
        """Is advice asked to p_groupId addable on item?"""
    def _adviceIsAddableByCurrentUser(groupId):
        """Even if adviceInfo['advice_addable'], is current user really able to add the advice?
           This is useful when using custom workflows and made to ease override of
           MeetingItem.getAdvicesGroupsInfosForUser."""
    def _adviceIsEditable(groupId):
        """Is advice asked to p_groupId editable on item?"""
    def _adviceIsEditableByCurrentUser(org_uid):
        """Even if adviceInfo['advice_editable'], is current user really able to edit the advice?
           This is useful when using custom workflows and made to ease override of
           MeetingItem.getAdvicesGroupsInfosForUser
           By default it checks if current user has ModifyPortalContent on advice object."""
    def _adviceDelayMayBeStarted(org_uid):
        """May the advice delay be started for p_org_uid, so may the 'delay_started_on' information be set?"""
    def _sendAdviceToGiveToGroup(org_uid):
        """Send the 'your advice is asked on this item' mail notification to given p_org_uid?"""
    def _advicePortalTypeForAdviser(org_uid):
        """Advices may use several 'meetingadvice' portal_types.  A portal_type is associated to
           an adviser org_uid, this method will return the advice portal_type used by given p_org_uid."""
    def _adviceDelayWillBeReinitialized(self, org_uid, adviceInfo, isTransitionReinitializingDelays):
        """Will advice delay be reinitialized for given p_ord_uid?
           By default delay is reinitialized if p_isTransitionReinitializingDelays,
           but if advice was not given and/or advice delay was not timed out."""
    def extraItemEvents(self):
        """Method for defining extra item events, needs to return a list of
           ids that will be used for id and translated for title."""
    def extraMeetingEvents(self):
        """Method for defining extra meeting events, needs to return a list of
           ids that will be used for id and translated for title."""
    def extraInsertingMethods(self):
        """Method for defining extra inserting methods, needs to return an OrderedDict
           where key is the inserting_method id and value, a config.ITEM_INSERT_METHODS compliant
           value that is used in the @@display-inserting-methods-helper-msg view."""
    def showAdvices(self):
        """This controls if advices need to be shown on the item view."""
    def _may_update_item_reference(self):
        """Condition to update item reference.  By default the item reference
           will be updated if item is in a meeting and meeting review_state is
           not 'before frozen'."""
    def _getGroupManagingItem(self, review_state, theObject=False):
        """Returns the group managing the item.
           By default this will be the proposingGroup.
           Given p_review_state may be used to know what group manage item in which review_state.
           This method must return an organization UID (or organization when theObject=True)."""
    def _getAllGroupsManagingItem(self, review_state, theObjects=False):
        """Returns the list of groups that manages the item until given p_review_state.
           This method must return a list of organizations UID (or organizations when theObjects=True).
           See _getGroupManagingItem docstring for more informations."""
    def custom_validate_optionalAdvisers(value, storedOptionalAdvisers, removedAdvisers):
        '''This is called by MeetingItem.validate_optionalAdvisers and let
           a plugin validates selected optional advisers.'''
    def getAdviceRelatedIndexes(self):
        '''Return item indexes related to advices, by default
           only the 'indexAdvisers' index is returned.'''
    def getReviewStateRelatedIndexes(self, check_deferred=True):
        '''Return item indexes related to review_state.'''
    def getIndexesRelatedTo(self, related_to='annex', check_deferred=True):
        '''Return catalog indexes related to given p_related_to and
           manage defering SearchableText reindex.'''
    def show_votesObservations():
        '''Votes observations field is only viewable by MeetingManagers and
           power observers until item is decided, in this case everybody may see it.'''
    def _bypass_meeting_closed_check_for(self, fieldName):
        """Adaptable method that let bypass the
           mayQuickEdit.bypassMeetingClosedCheck for given p_fieldName."""
    def _annex_decision_addable_states_after_validation(self, cfg):
        """Return states in which annex decision are addable in the WF states after
           the validation process (so when item is validated and after).
           By default this will be when item is decided."""
    def _assign_roles_to_non_managing_proposing_group_suffixes(self,
                                                               cfg,
                                                               item_state,
                                                               proposing_group_uid,
                                                               org_uid):
        """In case the group currently managing the item is not the proposingGroup,
           by default every suffixes of the proposingGroup will have the "Reader"
           role on the item so it may see it."""


class IMeetingItemWorkflowConditions(Interface):
    '''Conditions that may be defined in the workflow associated with a meeting
       item are defined as methods in this interface.'''
    def mayPropose():
        '''May this item be proposed by a member to some reviewer ?'''
    def mayPrevalidate():
        '''May this item be pre-validated by a pre-reviewer ?
           [only relevant when workflow adaptation "pre-validation" is
           enabled].'''
    def mayValidate():
        '''May this item be validated by a reviewer and proposed to a meeting
           manager ?'''
    def mayPresent():
        '''May this item be presented in a meeting ?'''
    def mayDecide():
        '''May a decision take place on this item (accept, refuse, delay, ...)?'''
    def mayCorrect(destinationState=None):
        '''Used for 'back' transitions.  p_destinationState is useful when there are
           several 'back' transitions from the same state.'''
    def mayPublish():
        '''May one publish me?'''
    def mayFreeze():
        '''May one freeze me ?'''
    def isLateFor(meeting):
        '''Normally, when meeting agendas are published (and seen by everyone),
           we shouldn't continue to add items to it. But sometimes those things
           need to happen :-). This method allows to determine under which
           circumstances an item may still be "late-presented" to a p_meeting.

           Here is the default behaviour of this method as implemented into
           PloneMeeting: an item whose preferred meeting is p_meeting, and
           that was validated after the p_meeting has been published, may still
           be presented to the p_meeting if the meeting is still in "published"
           state (so in this case, m_isLateFor returns True).

           Note that when such items are presented into a meeting, they are
           added in a special section, below the items that were presented under
           "normal" circumstances. This way, people that consult meeting agendas
           know that there is a fixed part of items that were in the meeting
           when it was first published, and that there are additional "late"
           items that were added in a hurry.'''
    def getListTypeNormalValue(meeting):
        '''Returns the normal value to set on item while presented to the p_meeting.
           By default this will be 'normal', but this is made to manage various 'normal-like'
           values.'''
    def getListTypeLateValue(meeting):
        '''Returns the late value to set on item while presented to the p_meeting.
           By default this will be 'late', but this is made to manage various 'late-like'
           values.'''


class IMeetingItemWorkflowActions(Interface):
    '''Actions that may be triggered while the workflow linked to an item
       executes.'''
    def doPropose(stateChange):
        '''Executes when an item is proposed to a reviewer.'''
    def doPrevalidate(stateChange):
        '''Executes when an item is pre-reviewed.'''
    def doValidate(stateChange):
        '''Executes when an action is validated by a reviewer and proposed to
           the meeting owner.'''
    def doPresent(stateChange):
        '''Executes when an item is presented in a meeting.'''
    def doItemPublish(stateChange):
        '''Executes when the meeting containing this item is published.'''
    def doItemFreeze(stateChange):
        '''Executes when the meeting containing this item is frozen (ie
           published, but without most people having the possibility to modify
           it).'''
    def doPre_accept(stateChange):
        '''Executes when an item is pre_accepted.'''
    def doAccept(stateChange):
        '''Executes when an item is accepted.'''
    def doAccept_but_modify(stateChange):
        '''Executes when an item is accepted_but_modified.'''
    def doRefuse(stateChange):
        '''Executes when an item is refused.'''
    def doDelay(stateChange):
        '''Executes when an item is delayed.'''
    def doCorrect(stateChange):
        '''Executes when the user performs a wrong action and needs to undo it.'''


class IMeetingItemCustom(IMeetingItem):
    '''If you want to propose your own implementations of IMeetingItem methods,
       you must define an adapter that adapts IMeetingItem to
       IMeetingItemCustom.'''


# Interfaces used for customizing the behaviour of meetings --------------------
class IMeetingDocumentation:
    '''Normally, the methods described here should be part of IMeeting.
       Because it is impossible to do so with an overengineered yet overrigid
       ArchGenXML 2, we document the provided methods in this absurd class.'''
    def is_decided():
        '''Am I in a state such that decisions have all been taken?'''
    def show_votes():
        '''Under what circumstances must I show the tab "Votes" for every item
           of this meeting? The default implementation for this method
           returns True when the meeting has started (based on meeting.date or
           meeting.startDate if used).'''
    def _check_insert_order_cache(cfg):
        '''This method is made to check if Meeting caching of items insert order
           is still valid.  Returns True if cache was invalidated, False otherwise.'''
    def _init_insert_order_cache(cfg):
        '''Initialize Meeting items insert order cache.'''
    def _insert_order_cache_cfg_attrs(cfg):
        '''Returns the field names of the MeetingConfig to take into account for
           Meeting items insert order caching.  If one of these fields value changed
           the cache would be invalidated.'''
    def get_late_state(self):
        '''Returns the meeting first review state from which presented items
           will be considered 'late'.'''
    def _may_update_item_references(self):
        """Condition to update items reference.  By default the item reference
           will be updated if meeting is late."""


class IMeetingWorkflowConditions(Interface):
    '''Conditions that may be defined in the workflow associated with a meeting
       are defined as methods in this interface.'''
    def mayPublish():
        '''May the user put me in a state where I am complete and I can be
           published and consulted by authorized persons before I begin?'''
    def mayFroze():
        '''May the user 'froze' the meeting? In this state, the meeting is
           published, is not decided yet but nobody may modify the meeting
           agenda anymore (at least in theory).'''
    def mayDecide():
        '''May the user put me in a state where all the decisions related to
           all my items are taken ?'''
    def mayClose():
        '''May the user put me in a state where all the decisions are completely
           finalized ?'''
    def mayArchive():
        '''May the user archive me ?'''
    def mayCorrect(destinationState=None):
        '''Used for 'back' transitions.  p_destinationState is useful when there are
           several 'back' transitions from the same state.'''
    def mayRepublish():
        '''May the user publish me again ? Returns False by default.'''

    # The following conditions are not workflow conditions in the strict sense,
    # but are conditions that depend on the meeting state.
    def may_accept_items():
        '''May I accept new items to be integrated to me ? (am I in a relevant
           state, is my date still in the future, ...)'''
    def may_change_items_order():
        '''May one change order of my list of items ?'''


class IMeetingWorkflowActions(Interface):
    '''Actions that may be triggered while the workflow linked to a meeting
       executes.'''
    def doPublish(stateChange):
        '''Executes when the meeting is "published" (=becomes visible by every
           authorized user). In the default PloneMeeting implementation,
           Meeting.doPublish calls Item.doPublish for every "presented" item
           contained in the meeting. It does so on the sorted list of items, so
           Item.doPublish methods are called in the item order. The default
           implementation also attributes a meeting number to the
           meeting (a sequence number within the meeting configuration).'''
    def doDecide(stateChange):
        '''Executes when all items contained in me are "decided". In the default
           PloneMeeting implementation, Meeting.doDecide calls Item.doAccept
           for every "frozen" item contained in the meeting.'''
    def doClose(stateChange):
        '''Executes when all decisions are finalized. In the default
           PloneMeeting implementation, Meeting.doClose calls Item.doConfirm
           for every "accepted" item contained in the meeting.'''
    def doArchive(stateChange):
        '''Executes when the meeting is archived.'''
    def doRepublish(stateChange):
        '''Executes when I am published again.'''
    def doBackToDecided(stateChange):
        '''Executes when I undo a "close" transition.'''
    def doBackToCreated(stateChange):
        '''Executes when I undo a "publish" transition.'''
    def doBackToPublished(stateChange):
        '''Executes when I undo a "decide" transition.'''
    def doBackToClosed(stateChange):
        '''Executes when I undo a "archive" transition.'''


class IMeetingCustom(IMeeting):
    '''If you want to propose your own implementations of IMeeting methods,
       you must define an adapter that adapts IMeeting to IMeetingCustom.'''
    def getIndexesRelatedTo(self, related_to='annex', check_deferred=True):
        '''Return catalog indexes related to given p_related_to and
           manage defering SearchableText reindex.'''


# Interfaces used for customizing the behaviour of meeting advice ----------
class IMeetingAdviceWorkflowConditions(Interface):
    '''Conditions that may be defined in the workflow associated with an advice
       are defined as methods in this interface.'''
    def mayGiveAdvice(self):
        '''Guard that protects the technical "giveAdvice" transition.'''
    def mayBackToAdviceInitialState(self):
        '''Guard that protects the technical "backToAdviceInitialState" transition.'''
    def mayCorrect(self, destinationState=None):
        '''Not used by default, a way to formalize use of mayCorrect to manage "back transitions".'''


class IMeetingAdviceWorkflowActions(Interface):
    '''Actions that may be triggered while the workflow linked to an advice executes.'''


# Interfaces used for customizing the behaviour of meeting configs -------------
# See docstring of previous classes for understanding this section.
class IMeetingConfigDocumentation:
    def custom_validate_workflowAdaptations(values, added, removed):
        '''This is called by MeetingConfig.validate_workflowAdaptations and let
           a plugin that added his own workflowAdaptations validates it.'''
    def onEdit(isCreated):
        '''Called when an object p_isCreated or edited.'''
    def _adviceConditionsInterfaceFor(self, advice_obj):
        '''Return the interface name to use to get the advice WF conditions adapter.'''
    def _adviceActionsInterfaceFor(self, advice_obj):
        '''Return the interface name to use to get the advice WF actions adapter.'''
    def get_item_corresponding_state_to_assign_local_roles(self, item_state):
        '''If an item_state is not managed by MeetingItem.assign_roles_to_group_suffixes,
           maybe there is a correspondence between current item_state and
           a managed item state.'''
    def get_item_custom_suffix_roles(self, item, item_state):
        """If an item_state is not managed by MeetingItem.assign_roles_to_group_suffixes,
           and no corresponding item state exists by default, we can manage
           suffix_roles manually."""
    def extra_item_decided_states(self):
        """Returns additional item decided states."""
    def extra_item_positive_decided_states(self):
        """Returns additional item positive decided states."""
    def _custom_reviewersFor(self):
        """Return an OrderedDict were key is the reviewer suffix and
           value the corresponding item state, from highest level to lower level.
           For example :
           OrderedDict([('reviewers', ['prevalidated']), ('prereviewers', ['proposed'])]).
           This must only be provided if the MeetingConfig.reviewersFor
           automatic computation is not correct."""
    def _custom_createOrUpdateGroups(self, force_update_access=False, dry_run_return_group_ids=False):
        """Method that will add custom MeetingConfig related Plone groups."""


class IMeetingConfigCustom(IMeetingConfig):
    pass


# Interfaces used for customizing the behaviour of meeting groups --------------
# See docstring of previous classes for understanding this section.
class IMeetingGroupDocumentation:
    def onEdit(isCreated):
        '''Called when an object p_isCreated or edited.'''


class IMeetingGroupCustom(IMeetingGroup):
    '''If you want to propose your own implementations of IMeetingGroup methods,
       you must define an adapter that adapts IMeetingGroup to
       IMeetingGroupCustom.'''


# Interfaces used for customizing the behaviour of the PloneMeeting tool -------
# See docstring of previous classes for understanding this section.
class IToolPloneMeetingDocumentation:
    def onEdit(isCreated):
        '''Called when the tool p_isCreated or edited.'''
    def performCustomWFAdaptations(meetingConfig, wfAdaptation, logger, itemWorkflow, meetingWorkflow):
        '''This let's a plugin define it's own WFAdaptations to apply.'''
    def get_extra_adviser_infos(self, group_by_org_uids=False):
        '''By default this will use data defined in ToolPloneMeeting.advisersConfig.
        But this can be overrided and return a similar format, each extra adviser
        infos must return a dict following information :
           - master key: an adviser organization uid, or a list of org uids when
             p_group_by_org_uids=True
           - value : a dict with :
               - 'portal_type' : the portal_type to use to give the advice;
               - 'base_wf' : the name of the base WF used by this portal_type;
                 will be used to generate a patched_ prefixed WF to apply WFAdaptations on;
               - 'wf_adaptations': a list of workflow adaptations to apply.
        '''
    def extraAdviceTypes(self):
        '''Method for defining extra advice types, needs to return a list of
           ids that will be used for id and translated for title.'''


class IToolPloneMeetingCustom(IToolPloneMeeting):
    '''If you want to propose your own implementations of tool methods,
       you must define an adapter that adapts IToolPloneMeeting to
       IToolPloneMeetingCustom.'''



# -*- coding: utf-8 -*-
#
# File: events.py
#
# Copyright (c) 2015 by Imio.be
# Generator: ArchGenXML Version 2.7
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

from Products.PloneMeeting.config import PloneMeetingError
from Products.PloneMeeting.tests.testViews import SAMPLE_ERROR_MESSAGE


def onItemListTypeChanged(item, event):
    '''Called when a MeetingItem.listType is changed.'''
    # raise a PloneMeetingError if title is already 'late - normal'
    if item.Title() == 'late - normal':
        raise PloneMeetingError(SAMPLE_ERROR_MESSAGE)
    # in this test, we just change item title and we
    # set old_listType and new listType
    item.setTitle("{0} - {1}".format(event.old_listType, item.getListType()))
    item.reindexObject()


def onItemLocalRolesUpdated(item, event):
    """Called after localRoles have been updated on the item.
       Add a local_role 'Reader' to user 'pmCreator2'."""
    item.manage_addLocalRoles('pmCreator2', ('Reader', ))


def onMeetingLocalRolesUpdated(item, event):
    """Called after localRoles have been updated on the meeting.
       Add a local_role 'MeetingManager' to user 'pmCreator2'."""
    item.manage_addLocalRoles('pmCreator2', ('MeetingManager', ))



# -*- coding: utf-8 -*-
#
# File: MeetingUser.py
#
# Copyright (c) 2015 by Imio.be
# Generator: ArchGenXML Version 2.7
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

from AccessControl import ClassSecurityInfo
from plone import api
from Products.Archetypes.atapi import AttributeStorage
from Products.Archetypes.atapi import BaseContent
from Products.Archetypes.atapi import BaseSchema
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import ImageField
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import MultiSelectionWidget
from Products.Archetypes.atapi import registerType
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import StringField
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
from Products.PloneMeeting.config import PROJECTNAME
from Products.PloneMeeting.config import WriteRiskyConfig
from Products.PloneMeeting.utils import getCustomAdapter
from zope.i18n import translate
from zope.interface import implements

import interfaces


schema = Schema((

    StringField(
        name='title',
        widget=StringField._properties['widget'](
            visible=False,
            label='Title',
            label_msgid='PloneMeeting_label_title',
            i18n_domain='PloneMeeting',
        ),
        accessor="Title",
        write_permission="PloneMeeting: Write risky config",
    ),
    StringField(
        name='configId',
        widget=StringField._properties['widget'](
            visible=False,
            label='Configid',
            label_msgid='PloneMeeting_label_configId',
            i18n_domain='PloneMeeting',
        ),
        write_permission="PloneMeeting: Write risky config",
    ),
    StringField(
        name='gender',
        widget=SelectionWidget(
            description="MeetingUserGender",
            description_msgid="meeting_user_gender_descr",
            condition="python: here.isManager()",
            label='Gender',
            label_msgid='PloneMeeting_label_gender',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
        vocabulary='listGenders',
    ),
    StringField(
        name='duty',
        widget=StringField._properties['widget'](
            description="MeetingUserDuty",
            description_msgid="meeting_user_duty_descr",
            condition="python: here.isManager()",
            label='Duty',
            label_msgid='PloneMeeting_label_duty',
            i18n_domain='PloneMeeting',
        ),
        write_permission="PloneMeeting: Write risky config",
    ),
    StringField(
        name='replacementDuty',
        widget=StringField._properties['widget'](
            description="MeetingUserReplacementDuty",
            description_msgid="meeting_user_replacement_duty_descr",
            condition="python: here.isManager()",
            label='Replacementduty',
            label_msgid='PloneMeeting_label_replacementDuty',
            i18n_domain='PloneMeeting',
        ),
        write_permission="PloneMeeting: Write risky config",
    ),
    LinesField(
        name='usages',
        widget=MultiSelectionWidget(
            description="MeetingUserUsages",
            description_msgid="meeting_user_usages_descr",
            format="checkbox",
            condition="python: here.isManager()",
            label='Usages',
            label_msgid='PloneMeeting_label_usages',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary='listUsages',
        write_permission="PloneMeeting: Write risky config",
    ),
    ImageField(
        name='signatureImage',
        widget=ImageField._properties['widget'](
            description="MeetingUserSignatureImage",
            description_msgid="meeting_user_signature_image_descr",
            condition="python: here.isManager()",
            label='Signatureimage',
            label_msgid='PloneMeeting_label_signatureImage',
            i18n_domain='PloneMeeting',
        ),
        storage=AttributeStorage(),
        write_permission="PloneMeeting: Write risky config",
    ),
    BooleanField(
        name='signatureIsDefault',
        default=False,
        widget=BooleanField._properties['widget'](
            description="MeetingUserSignatureIsDefault",
            description_msgid="meeting_user_signature_is_default",
            condition="python: here.isManager()",
            label='Signatureisdefault',
            label_msgid='PloneMeeting_label_signatureIsDefault',
            i18n_domain='PloneMeeting',
        ),
        write_permission="PloneMeeting: Write risky config",
    ),
    StringField(
        name='adviceStyle',
        widget=SelectionWidget(
            description="AdviceStyle",
            description_msgid="advice_style_descr",
            label='Advicestyle',
            label_msgid='PloneMeeting_label_adviceStyle',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
        vocabulary='listAdviceStyles',
        default_method="getDefaultAdviceStyle",
    ),
    LinesField(
        name='itemsListVisibleColumns',
        widget=MultiSelectionWidget(
            description="ItemsListVisibleColumns",
            description_msgid="items_list_visible_columns_descr",
            format="checkbox",
            label='Itemslistvisiblecolumns',
            label_msgid='PloneMeeting_label_itemsListVisibleColumns',
            i18n_domain='PloneMeeting',
        ),
        multiValued=1,
        vocabulary='listItemsListVisibleColumns',
        default_method="getDefaultVisibleColumns",
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
    ),
    LinesField(
        name='itemColumns',
        widget=MultiSelectionWidget(
            description="ItemColumns",
            description_msgid="item_columns_descr",
            format="checkbox",
            label='Itemcolumns',
            label_msgid='PloneMeeting_label_itemColumns',
            i18n_domain='PloneMeeting',
        ),
        multiValued=1,
        vocabulary="listItemColumns",
        default_method="getDefaultItemColumns",
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
    ),
    LinesField(
        name='meetingColumns',
        widget=MultiSelectionWidget(
            description="MeetingColumns",
            description_msgid="meeting_columns_descr",
            format="checkbox",
            label='Meetingcolumns',
            label_msgid='PloneMeeting_label_meetingColumns',
            i18n_domain='PloneMeeting',
        ),
        multiValued=1,
        vocabulary="listMeetingColumns",
        default_method="getDefaultMeetingColumns",
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
    ),
    LinesField(
        name='mailItemEvents',
        widget=MultiSelectionWidget(
            description="MailItemEvents",
            description_msgid="mail_item_events_descr",
            condition="python: here.listItemEvents()",
            format="checkbox",
            label='Mailitemevents',
            label_msgid='PloneMeeting_label_mailItemEvents',
            i18n_domain='PloneMeeting',
        ),
        multiValued=1,
        vocabulary="listItemEvents",
        default_method="getDefaultItemEvents",
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
    ),
    LinesField(
        name='mailMeetingEvents',
        widget=MultiSelectionWidget(
            description="MailMeetingEvents",
            description_msgid="mail_meeting_events",
            condition="python: here.listMeetingEvents()",
            format="checkbox",
            label='Mailmeetingevents',
            label_msgid='PloneMeeting_label_mailMeetingEvents',
            i18n_domain='PloneMeeting',
        ),
        multiValued=1,
        vocabulary="listMeetingEvents",
        default_method="getDefaultMeetingEvents",
        enforceVocabulary=True,
        write_permission="PloneMeeting: Write risky config",
    ),

),
)

MeetingUser_schema = BaseSchema.copy() + \
    schema.copy()

MeetingUser_schema['id'].write_permission = "PloneMeeting: Write risky config"
MeetingUser_schema['title'].write_permission = "PloneMeeting: Write risky config"
# hide metadata fields and even protect it vy the WriteRiskyConfig permission
for field in MeetingUser_schema.getSchemataFields('metadata'):
    field.widget.visible = {'edit': 'invisible', 'view': 'invisible'}
    field.write_permission = WriteRiskyConfig


class MeetingUser(BaseContent, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IMeetingUser)

    meta_type = 'MeetingUser'
    _at_rename_after_creation = True

    schema = MeetingUser_schema

    security.declarePublic('getSelf')

    def getSelf(self):
        if self.getTagName() != 'MeetingUser':
            return self.context
        return self

    security.declarePublic('adapted')

    def adapted(self):
        return getCustomAdapter(self)

    security.declareProtected('Modify portal content', 'onEdit')

    def onEdit(self, isCreated):
        '''See doc in interfaces.py.'''
        pass

    def query_state(self):
        '''In what state am I ?'''
        wfTool = api.portal.get_tool('portal_workflow')
        return wfTool.getInfoFor(self, 'review_state')

    security.declarePublic('config')

    def config(self):
        return self.getParentNode().getParentNode()

    security.declarePrivate('updateMeetingUser')

    def updateMeetingUser(self):
        '''Updates this meeting user (local roles, title).'''
        # Updates the title
        membershipTool = api.portal.get_tool('portal_membership')
        userInfo = membershipTool.getMemberById(self.id)
        if userInfo and userInfo.getProperty('fullname'):
            title = userInfo.getProperty('fullname')
        else:
            title = self.id
        self.setTitle(title)
        self.setConfigId(self.config().id)
        # Update local roles. Corresponding Plone user owns me.
        if 'Owner' not in self.get_local_roles_for_userid(self.id):
            self.manage_addLocalRoles(self.id, ('Owner',))

    security.declarePrivate('at_post_create_script')

    def at_post_create_script(self):
        self.updateMeetingUser()
        self.adapted().onEdit(isCreated=True)

    security.declarePrivate('at_post_edit_script')

    def at_post_edit_script(self):
        self.updateMeetingUser()
        self.adapted().onEdit(isCreated=False)

    def listGenders(self):
        '''Lists the genders (M, F).'''
        res = DisplayList((
            ("m", translate('gender_m', domain='PloneMeeting', context=self.REQUEST)),
            ("f", translate('gender_f', domain='PloneMeeting', context=self.REQUEST)),
        ))
        return res

    def listUsages(self):
        '''Returns list of possible usages (for what will this user be useful
           in voting process: "assembly member", "signer" or "voter").'''
        d = 'PloneMeeting'
        res = DisplayList((
            ("assemblyMember", translate('meeting_user_usage_assemblyMember', domain=d, context=self.REQUEST)),
            ("signer", translate('meeting_user_usage_signer', domain=d, context=self.REQUEST)),
            ("voter", translate("meeting_user_usage_voter", domain=d, context=self.REQUEST)),
            ("asker", translate("meeting_user_usage_asker", domain=d, context=self.REQUEST)),
        ))
        return res

    security.declarePublic('mayConsultVote')

    def mayConsultVote(self, loggedUser, item):
        '''See doc in interfaces.py.'''
        mUser = self.getSelf()
        tool = getToolByName(mUser, 'portal_plonemeeting')
        if (loggedUser.id == mUser.getId()) or \
           tool.isManager(item) or \
           item.getMeeting().adapted().isDecided():
            return True
        return False

    security.declarePublic('mayEditVote')

    def mayEditVote(self, loggedUser, item):
        '''See doc in interfaces.py.'''
        mUser = self.getSelf()
        tool = getToolByName(item, 'portal_plonemeeting')
        if loggedUser.has_role('Manager'):
            return True
        meeting = item.getMeeting()
        if item.getMeeting().query_state() in meeting.meetingClosedStates:
            return False
        else:
            cfg = tool.getMeetingConfig(item)
            votesEncoder = cfg.getVotesEncoder()
            if (loggedUser.id == mUser.getId()) and \
               ('theVoterHimself' in votesEncoder):
                return True
            if tool.isManager(item) and \
               ('aMeetingManager' in votesEncoder):
                return True
        return False

    security.declarePublic('isManager')

    def isManager(self):
        '''Has logged user role Manager?'''
        membershipTool = api.portal.get_tool('portal_membership')
        user = membershipTool.getAuthenticatedMember()
        return user.has_role('Manager')

    def listAdviceStyles(self):
        return self.config().listAdviceStyles()

    def getDefaultAdviceStyle(self):
        return self.config().getAdviceStyle()

    def listItemsListVisibleColumns(self):
        return self.config().listItemsListVisibleColumns()

    def getDefaultVisibleColumns(self):
        return self.config().getItemsListVisibleColumns()

    def listItemColumns(self):
        return self.config().listItemColumns()

    def getDefaultItemColumns(self):
        return self.config().getItemColumns()

    def listMeetingColumns(self):
        return self.config().listMeetingColumns()

    def getDefaultMeetingColumns(self):
        return self.config().getMeetingColumns()

    def listItemEvents(self):
        config = self.config()
        allEvents = config.listItemEvents()
        selectedEvents = config.getMailItemEvents()
        # From the list of all events that are selectable in the config
        # (=configItemEvents), the user may only (de)activate events
        # that are selected in the config (=selectedEvents). Among
        # selectedEvents, the user may only (de)activate events that make sense
        # (ie, if the user is not an adviser, it has no sense to propose him to
        # (de)activate events sent to an adviser).
        res = []
        # We must know if the user is a MeetingManager, an item creator or
        # adviser
        tool = config.getParentNode()
        isMeetingManager = tool.isManager(config)
        isCreator = tool.userIsAmong(['creators'])
        isAdviser = tool.userIsAmong(['advisers'])
        for event in selectedEvents:
            keepIt = False
            if (event in ('lateItem', 'annexAdded', 'askDiscussItem')) and \
               isMeetingManager:
                keepIt = True
            elif event in ('itemPresented', 'itemUnpresented', 'itemDelayed', 'itemClonedToThisMC') and isCreator:
                keepIt = True
            elif event in ('adviceToGive', 'adviceInvalidated') and isAdviser:
                keepIt = True
            elif event == 'adviceEdited':
                keepIt = True  # This event requires permission "View".
            if keepIt:
                res.append((event, allEvents.getValue(event)))
        return DisplayList(tuple(res))

    def getDefaultItemEvents(self):
        return self.listItemEvents().keys()

    def listMeetingEvents(self):
        config = self.config()
        allEvents = config.listMeetingEvents()
        # On an archive site, there may be no transition defined on a meeting.
        if not allEvents.keys():
            return DisplayList()
        selectedEvents = config.getMailMeetingEvents()
        res = []
        # Here, I could only keep transitions for which the user has the
        # permission to View the meeting at its end state. But in most cases,
        # meeting-related events can be seen by everyone.
        for event in selectedEvents:
            eventValue = allEvents.getValue(event)
            if not eventValue:
                continue
            res.append((event, eventValue))
        return DisplayList(tuple(res))

    def getDefaultMeetingEvents(self):
        return self.listMeetingEvents().keys()

    def getForUseIn(self, meeting):
        '''Gets the user as playing a role in p_meeting, taking into account
           possible user replacements.'''
        if not hasattr(meeting.aq_base, 'userReplacements'):
            return self
        # cfg = meeting.portal_plonemeeting.getMeetingConfig(meeting)
        # Is this user replaced by another user ?
        if self.getId() in meeting.userReplacements:
            # Yes it is. Find the replacement.
            # repl = getattr(cfg.meetingusers,
            #                meeting.userReplacements[self.getId()])
            # Return the replacement user instead of this user, with adapted
            # characteristics, in the form of a FakeMeetingUser instance.
            return None
        # Is this user a replacement for another user ?
        for baseUser, replUser in meeting.userReplacements.iteritems():
            if self.getId() != replUser:
                continue
            # Yes it is. Find the replaced person.
            # repl = getattr(cfg.meetingusers, baseUser)
            # Return this user (self), but with adapted characteristics
            # (because he replaces someone else), in the form of a
            # FakeMeetingUser instance.
            return None
        return self

    security.declarePublic('isPresent')

    def isPresent(self, item, meeting):
        '''Is this user present at p_meeting when p_item is discussed?'''
        aId = self.getId()
        if aId in item.get_item_absents():
            return False
        if aId in meeting.getLateAttendees():
            entranceNumber = meeting.getEntranceItem(aId)
            if not entranceNumber or \
               (entranceNumber > item.getItemNumber(relativeTo='meeting')):
                return False
        if aId in meeting.getDepartures(item, when='before', alsoEarlier=True):
            return False
        return True

    security.declarePublic('indexUsages')

    def indexUsages(self):
        '''Returns the index content for usages.'''
        return self.getUsages()


registerType(MeetingUser, PROJECTNAME)



# -*- coding: utf-8 -*-

from collective.ckeditor.browser.ckeditorview import CKeditorView


class PMCKeditorView(CKeditorView):
    """ """

    def getCK_plone_config(self):
        """XXX temporary fix to have a correct URL when @@cke-upload-image is called."""
        res = super(PMCKeditorView, self).getCK_plone_config()
        # include context absolute_url when calling @@cke-upload-image so it works
        # when editing an advice from a faceted dashboard
        if "/@@cke-upload-image" not in res:
            res = res.replace(
                "@@cke-upload-image",
                "%s/@@cke-upload-image" % self.context.absolute_url())
        return res
